@@ -25,6 +25,7 @@
 
 #include "error.h"
 #include "filebuf.h"
+#include "render.h"
 #include "table.h"
 #include "text.h"
 #include "textset.h"
@@ -55,35 +56,25 @@ struct string_arg {
 
 
 static struct string_arg char_maps[] = {
-	{ "case", CORPUS_TYPE_CASEFOLD,
+	{ "case", CORPUS_TYPE_MAPCASE,
 		"Performs Unicode case-folding." },
-	{ "control", CORPUS_TYPE_RMCC,
-		"Remove non-white-space control characters."},
-	{ "compat", CORPUS_TYPE_COMPAT,
+	{ "compat", CORPUS_TYPE_MAPCOMPAT,
 		"Applies Unicode compatibility mappings."},
-	{ "dash", CORPUS_TYPE_DASHFOLD,
-		"Replaces Unicode dashes with ASCII dash (-)." },
 	{ "ignorable", CORPUS_TYPE_RMDI,
 		"Removes Unicode default ignorables." },
-	{ "quote", CORPUS_TYPE_QUOTFOLD,
+	{ "quote", CORPUS_TYPE_MAPQUOTE,
 		"Replaces Unicode quotes with ASCII single quote (')." },
-	{ "space", CORPUS_TYPE_RMWS,
-		"Remove white-space characters."},
 	{ NULL, 0, NULL }
 };
 
 
 static struct string_arg word_classes[] = {
-	{ "symbol", CORPUS_FILTER_DROP_SYMBOL,
-		"Does not fit into any other category." },
-	{ "number", CORPUS_FILTER_DROP_NUMBER,
-		"Appears to be a number." },
-	{ "letter", CORPUS_FILTER_DROP_LETTER,
-		"Composed of letters (not kana or ideographic)." },
-	{ "kana", CORPUS_FILTER_DROP_KANA,
-		"Composed of kana characters." },
-	{ "ideo", CORPUS_FILTER_DROP_IDEO,
-		"Composed of ideographic characters."},
+	{ "letter", CORPUS_FILTER_DROP_LETTER, "Composed of letters." },
+	{ "mark",   CORPUS_FILTER_DROP_MARK,   "Marks." },
+	{ "number", CORPUS_FILTER_DROP_NUMBER, "Appears to be a number." },
+	{ "punct",  CORPUS_FILTER_DROP_PUNCT,  "Punctuation." },
+	{ "symbol", CORPUS_FILTER_DROP_SYMBOL, "Symbols." },
+	{ "other",  CORPUS_FILTER_DROP_OTHER,  "Other." },
 	{ NULL, 0, NULL }
 };
 
@@ -121,7 +112,7 @@ Options:\n\
 \t-o <path>\tSaves output at the given path.\n\
 \t-s <stemmer>\tStems tokens with the given algorithm.\n\
 \t-t <stopwords>\tDrops words from the given stop word list.\n\
-\t-z\t\tKeeps zero-character (empty) tokens.\n\
+\t-z\t\tKeeps white space tokens.\n\
 ", PROGRAM_NAME);
 	printf("\nCharacter Maps:\n");
 	for (i = 0; char_maps[i].name != NULL; i++) {
@@ -177,10 +168,11 @@ int main_tokens(int argc, char * const argv[])
 	struct corpus_filter filter;
 	struct corpus_data data, val;
 	struct corpus_text name, text, word;
-	const struct corpus_text *term;
+	const struct corpus_text *type;
 	struct corpus_schema schema;
 	struct corpus_filebuf buf;
 	struct corpus_filebuf_iter it;
+	struct corpus_render render;
 	const char *output = NULL;
 	const char *stemmer = NULL;
 	const uint8_t **stopwords = NULL;
@@ -188,13 +180,11 @@ int main_tokens(int argc, char * const argv[])
 	FILE *stream;
 	size_t field_len;
 	int filter_flags, type_flags;
-	int ch, err, i, name_id, start, term_id, ncomb;
+	int ch, err, i, name_id, start, type_id, ncomb;
 
-	filter_flags = CORPUS_FILTER_IGNORE_EMPTY;
-	type_flags = (CORPUS_TYPE_COMPAT | CORPUS_TYPE_CASEFOLD
-			| CORPUS_TYPE_DASHFOLD | CORPUS_TYPE_QUOTFOLD
-			| CORPUS_TYPE_RMCC | CORPUS_TYPE_RMDI
-			| CORPUS_TYPE_RMWS);
+	filter_flags = CORPUS_FILTER_IGNORE_SPACE;
+	type_flags = (CORPUS_TYPE_MAPCASE | CORPUS_TYPE_MAPCOMPAT
+			| CORPUS_TYPE_MAPQUOTE | CORPUS_TYPE_RMDI);
 
 	field = "text";
 	ncomb = 0;
@@ -244,7 +234,7 @@ int main_tokens(int argc, char * const argv[])
 			stemmer = optarg;
 			break;
 		case 't':
-			if (!(stopwords = corpus_stopwords(optarg, NULL))) {
+			if (!(stopwords = corpus_stopword_list(optarg, NULL))) {
 				fprintf(stderr,
 					"Unrecognized stop word list: '%s'."
 					"\n\n", optarg);
@@ -253,7 +243,7 @@ int main_tokens(int argc, char * const argv[])
 			}
 			break;
 		case 'z':
-			filter_flags &= ~CORPUS_FILTER_IGNORE_EMPTY;
+			filter_flags &= ~CORPUS_FILTER_IGNORE_SPACE;
 			break;
 		default:
 			usage_tokens();
@@ -287,6 +277,11 @@ int main_tokens(int argc, char * const argv[])
 		return EXIT_FAILURE;
 	}
 
+	if ((err = corpus_render_init(&render, (CORPUS_ESCAPE_CONTROL
+						| CORPUS_ESCAPE_UTF8)))) {
+		goto error_render;
+	}
+
 	if ((err = corpus_schema_init(&schema))) {
 		goto error_schema;
 	}
@@ -378,13 +373,15 @@ int main_tokens(int argc, char * const argv[])
 		fprintf(stream, "[");
 		start = 1;
 
-		if ((err = corpus_filter_start(&filter, &text))) {
+		if ((err = corpus_filter_start(&filter, &text,
+					       CORPUS_FILTER_SCAN_TOKENS))) {
 			goto error;
 		}
 
-		while (corpus_filter_advance(&filter, &term_id)) {
-			if (filter.error) {
-				goto error;
+		while (corpus_filter_advance(&filter)) {
+			type_id = filter.type_id;
+			if (type_id == CORPUS_FILTER_IGNORED) {
+				continue;
 			}
 
 			if (!start) {
@@ -393,16 +390,24 @@ int main_tokens(int argc, char * const argv[])
 				start = 0;
 			}
 
-			if (term_id < 0) {
+			if (type_id < 0) {
 				fprintf(stream, "null");
 			} else {
-				term = corpus_filter_term(&filter, term_id);
+				type = corpus_filter_type(&filter, type_id);
+
+				corpus_render_clear(&render);
+				corpus_render_text(&render, type);
+				if ((err = render.error)) {
+					goto error;
+				}
 
 				fprintf(stream, "\"%.*s\"",
-					(int)CORPUS_TEXT_SIZE(term),
-					(char *)term->ptr);
+					render.length, render.string);
 			}
 		}
+		if (filter.error) {
+			goto error;
+		}
 		fprintf(stream, "]\n");
 	}
 
@@ -421,6 +426,8 @@ error_stopwords:
 error_filter:
 	corpus_schema_destroy(&schema);
 error_schema:
+	corpus_render_destroy(&render);
+error_render:
 	if (err) {
 		fprintf(stderr, "An error occurred.\n");
 		return EXIT_FAILURE;
