@@ -18,6 +18,7 @@
 #include <stdint.h>
 #include <stdlib.h>
 #include "unicode/casefold.h"
+#include "unicode/charwidth.h"
 #include "unicode/compose.h"
 #include "unicode/combining.h"
 #include "unicode/decompose.h"
@@ -253,6 +254,31 @@ void corpus_encode_utf8(uint32_t code, uint8_t **bufptr)
 }
 
 
+void corpus_rencode_utf8(uint32_t code, uint8_t **bufptr)
+{
+	uint8_t *ptr = *bufptr;
+	uint32_t x = code;
+
+	if (x <= 0x7F) {
+		*--ptr = (uint8_t)x;
+	} else if (x <= 0x07FF) {
+		*--ptr = (uint8_t)(0x80 | (x & 0x3F));
+		*--ptr = (uint8_t)(0xC0 | (x >> 6));
+	} else if (x <= 0xFFFF) {
+		*--ptr = (uint8_t)(0x80 | (x & 0x3F));
+		*--ptr = (uint8_t)(0x80 | ((x >> 6) & 0x3F));
+		*--ptr = (uint8_t)(0xE0 | (x >> 12));
+	} else {
+		*--ptr = (uint8_t)(0x80 | (x & 0x3F));
+		*--ptr = (uint8_t)(0x80 | ((x >> 6) & 0x3F));
+		*--ptr = (uint8_t)(0x80 | ((x >> 12) & 0x3F));
+		*--ptr = (uint8_t)(0xF0 | (x >> 18));
+	}
+
+	*bufptr = ptr;
+}
+
+
 /* From Unicode-8.0 Section 3.12 Conjoining Jamo Behavior */
 #define HANGUL_SBASE 0xAC00
 #define HANGUL_LBASE 0x1100
@@ -603,3 +629,26 @@ void corpus_unicode_compose(uint32_t *ptr, size_t *lenptr)
 out:
 	*lenptr = len;
 }
+
+
+int corpus_unicode_charwidth(uint32_t code)
+{
+	int prop = charwidth(code);
+	switch(prop) {
+	case CHARWIDTH_OTHER:
+		return CORPUS_CHARWIDTH_OTHER;
+	case CHARWIDTH_AMBIGUOUS:
+		return CORPUS_CHARWIDTH_AMBIGUOUS;
+	case CHARWIDTH_IGNORABLE:
+		return CORPUS_CHARWIDTH_IGNORABLE;
+	case CHARWIDTH_NONE:
+		return CORPUS_CHARWIDTH_NONE;
+	case CHARWIDTH_NARROW:
+		return CORPUS_CHARWIDTH_NARROW;
+	case CHARWIDTH_WIDE:
+		return CORPUS_CHARWIDTH_WIDE;
+	default:
+		assert(0 && "internal error: unrecognized charwidth property");
+		return CORPUS_CHARWIDTH_OTHER;
+	}
+}
