@@ -34,10 +34,11 @@ static const R_CallMethodDef CallEntries[] = {
 	CALLDEF(as_logical_json, 1),
 	CALLDEF(as_text_character, 1),
 	CALLDEF(as_text_json, 1),
-	CALLDEF(datatype_json, 1),
-	CALLDEF(datatypes_json, 1),
 	CALLDEF(dim_json, 1),
+	CALLDEF(format_text, 6),
 	CALLDEF(is_na_text, 1),
+	CALLDEF(json_datatype, 1),
+	CALLDEF(json_datatypes, 1),
 	CALLDEF(length_json, 1),
 	CALLDEF(length_text, 1),
 	CALLDEF(logging_off, 0),
@@ -51,11 +52,18 @@ static const R_CallMethodDef CallEntries[] = {
 	CALLDEF(subscript_json, 2),
 	CALLDEF(subset_json, 3),
 	CALLDEF(subset_text_handle, 2),
-	CALLDEF(tokens_text, 2),
 	CALLDEF(term_counts_text, 7),
 	CALLDEF(term_matrix_text, 6),
-	CALLDEF(text_split_sentences, 4),
+	CALLDEF(text_count, 3),
+	CALLDEF(text_detect, 3),
+	CALLDEF(text_locate, 3),
+	CALLDEF(text_nsentence, 2),
+	CALLDEF(text_ntoken, 2),
+	CALLDEF(text_ntype, 3),
+	CALLDEF(text_split_sentences, 3),
 	CALLDEF(text_split_tokens, 3),
+	CALLDEF(text_tokens, 2),
+	CALLDEF(text_types, 3),
         {NULL, NULL, 0}
 };
 
