@@ -2,13 +2,13 @@ context("text_names")
 
 
 test_that("`names` should be NULL for new text", {
-    x <- text("ABC")
+    x <- as_text("ABC")
     expect_equal(names(x), NULL)
 })
 
 
 test_that("`names<-` should work on text", {
-    x <- text(LETTERS)
+    x <- as_text(LETTERS)
     names(x) <- rev(LETTERS)
     expect_equal(names(x), rev(LETTERS))
 })
@@ -17,14 +17,11 @@ test_that("`names<-` should work on text", {
 test_that("`as_text` should not drop names", {
     x <- as_text(c(a="1", b="2"))
     expect_equal(names(x), c("a", "b"))
-
-    y <- as_text(text(a="1", b="2"))
-    expect_equal(names(y), c("a", "b"))
 })
 
 
 test_that("`all.equal` should test names", {
-    x <- text(1:3)
+    x <- as_text(1:3)
     y <- x
     names(y) <- c("a", "b", "c")
     expect_equal(all.equal(x, y), "names for current but not for target")
@@ -33,19 +30,19 @@ test_that("`all.equal` should test names", {
 
 
 test_that("`as_text` should not drop names", {
-    x <- text(foo="hello")
+    x <- as_text(c(foo="hello"))
     y <- as_text(x)
 
-    expect_equal(y, text(foo="hello"))
+    expect_equal(y, as_text(c(foo="hello")))
 })
 
 
 test_that("`as_text` should drop attributes", {
-    x <- text("hello")
+    x <- as_text("hello")
     attr(x, "foo") <- "bar"
     y <- as_text(x)
 
-    expect_equal(y, text("hello"))
+    expect_equal(y, as_text("hello"))
 })
 
 
@@ -57,12 +54,12 @@ test_that("`as_text` should drop attributes for JSON objects", {
     attr(x, "foo") <- "bar"
     y <- as_text(x)
 
-    expect_equal(y, text("hello"))
+    expect_equal(y, as_text("hello"))
 })
 
 
 test_that("`names<-` should not modify copies", {
-    x <- text(1:3)
+    x <- as_text(1:3)
     y <- x
     names(y) <- c("a", "b", "c")
     expect_equal(names(x), NULL)
@@ -71,7 +68,7 @@ test_that("`names<-` should not modify copies", {
 
 
 test_that("`names<-` should preserve attributes", {
-    x <- text(1:3)
+    x <- as_text(1:3)
     attr(x, "foo") <- "bar"
     names(x) <- c("a", "b", "c")
     expect_equal(names(x), c("a", "b", "c"))
@@ -80,14 +77,14 @@ test_that("`names<-` should preserve attributes", {
 
 
 test_that("`names<-` should allow NA", {
-    x <- text(1:3)
+    x <- as_text(1:3)
     names(x) <- c("a", NA, "b")
     expect_equal(names(x), c("a", NA, "b"))
 })
 
 
 test_that("`names<-` should allow duplicates", {
-    x <- text(1:3)
+    x <- as_text(1:3)
     names(x) <- c("a", "b", "a")
     expect_equal(names(x), c("a", "b", "a"))
 })
